/*
	JTyphoon
		気象庁発表の台風進路予報図を表示する Dashboard Widget

		このソフトウェアは「現状のまま」何らの保証もなく提供されます。 
		このソフトウェアはパブリック・ドメイン（Public Domain）に置かれており、
		作者はこのソフトウェアに関して著作権を主張しません。改変、再配布等はご自由にどうぞ。 

		http://www2.atwiki.jp/macstar/pages/17.html
*/

var jmaURI = "http://www.jma.go.jp/jp/typh/";
var widgetURI = "http://www2.atwiki.jp/macstar/pages/17.html";
var httpReq;
var typhNum;					// 地図を表示中の台風
var typhs = new Array();		// 台風の名前リスト
var typhInfos = new Array();	// 台風の詳細情報（未実装）
var lastGetTime;				// 表示に使っているページの Last-Modified
var lastCheckTime;				// 最後にリロードを試みた時刻

// リサイズ関連
var defaultWidth = 270;
var defaultHeight = 240;
var minWidth = 230;
var minHeight = 210;
var maxWidth = 510;
var maxHeight = 420;
var lastX = 0
var lastY = 0;

// 定期リロード関連（30分毎にリロード）
var lastCheckTime;
var timerID;
var interval = 30*60*1000;

// widget初期化
function load() {
	if (window.widget) {
		widget.onshow = onShow;
	}

	// XMLHttpRequest
	httpReq = new XMLHttpRequest();
	httpReq.onreadystatechange = processReqChange;

	typhNum = 0;
	typhs.length = 0;
	lastCheckTime = 0;
	lastGetTime = "Thu, 01 Jan 1970 00:00:00 GMT";
	mapDisp(typhNum);

	// 設定ファイルからウインドウサイズを読み出し
	setWidgetSize(widget.preferenceForKey("widgetWidth"), widget.preferenceForKey("widgetHeight"));

	// 初回の読み込み
	reload();

	// 定期リロード開始
	timerID = setInterval("reload()", interval);
}


// Dashboardレイヤー表示時
function onShow() {
	var now = (new Date).getTime();

	// スリープ対策
	// 前回リロード時からinterval-1秒以上経過していたらリロード
	if ((lastCheckTime != 0) &&
			(-1*1000 < (now - lastCheckTime - interval))) {
		reload();
	}
}


// 指定された番号の予報図を表示
function mapDisp(num) {
	var newNum = num;
	var i;

	// 指定された番号の台風が無くなっている場合の処理
	if (typhs.length < newNum)
		newNum = typhs.length - 1;

	for (i = 0; i < 8; i++) {
		if (i == newNum) {
			document.getElementById("map" + i).style.display = "block";
			if (document.getElementById("typh" + i))
				document.getElementById("typh" + i).style.opacity = 1.0;
		} else {
			document.getElementById("map" + i).style.display = "none";
			if (document.getElementById("typh" + i))
				document.getElementById("typh" + i).style.opacity = 0.3;
		}
	}
	typhNum = newNum;
}


// 最新の情報で再描画
function reload() {
	var uri = jmaURI + 'index.html';	// 台風情報トップページ
	
	// デバッグ用
	document.getElementById("status").innerText = "";
	document.getElementById("header").innerText = "";
	document.getElementById("source").innerText = "";

	// Dashboardが用意するキャッシュを使いたくないので（？）
	// 強制的にリロードさせるための小細工（URIに「?ユニークな文字列」を付加）
	uri += "?" + new Date().getTime();
	LOG__("Reload URI: " + uri );

	// HTTPリクエスト（非同期読み込み）
	// lastGetTimeよりも新しいコンテンツを要求する
	httpReq.open("GET", uri, true);
	if (0 < lastGetTime.length)
		httpReq.setRequestHeader("If-Modified-Since", lastGetTime);
	httpReq.send(null);
	
	lastCheckTime = (new Date).getTime();
}


// リクエスト状態（readyState）が変わったときに呼び出される
function processReqChange() {
	var httpHeader = "";
	var htmlDoc = "";

	LOG__("httpReq.readyState " + httpReq.readyState);

	// デバッグ用
	if (!(window.widget) && (httpReq.readyState != 0)) { // readyState 0 = uninitialized
		document.getElementById("status").innerText = httpReq.status;
		document.getElementById("header").innerText = httpReq.getAllResponseHeaders();
		document.getElementById("source").innerText = httpReq.responseText;
	}

	if (httpReq.readyState == 4) { // readyState 4 = complete
		if (httpReq.status == 200) {						// 200:ダウンロード成功
			httpHeader = httpReq.getAllResponseHeaders();	// レスポンスヘッダ
			htmlDoc = httpReq.responseText;					// HTMLソース
			document.getElementById("source").innerText = htmlDoc;

			// 最終更新時刻を記録
			if (httpHeader.match(/Last-Modified: (.+)/i))
				lastGetTime = RegExp.$1;
			else
				lastGetTime = "";

			// HTMLソースから台風リストを作成
			parseHTML(htmlDoc);
		}

		// 表示を更新
		dispTyphs();
		mapDisp(typhNum);
	}
}


// HTMLソースから台風リストを作成
function parseHTML(theHtml) {
	var buffer = "";
	var options = new Array();
	var head, tail;
	var i;
    var optReg = new RegExp("<option[^>]+>", "g");
    var valReg = new RegExp("value=\"(all|[0-9][0-9][0-9][0-9]|[a-z])\"");
    var typhCode;

	typhs.length = 0;

	// 台風の名前を select#typhoonSelect 要素から取得
    // 　　(例) all, 0901, 0902, ... , a, b, c, ...
	head = theHtml.indexOf("<select id=\"typhoonSelect\"");
	if (0 <= head)
		tail = theHtml.indexOf("</select>", head);
	if (0 <= tail)
		buffer  = theHtml.slice(head, tail);
	options = buffer.match(optReg);

	// 台風情報を取得できた場合
	if (options != null) {
		for (i = 0; i < options.length; i++) {
			if (typhCode = (options[i].match(valReg) || [])[1])
				typhs.push(typhCode);
		}
	}

	// 台風情報を取得できなかった場合
	if (typhs.length <= 0) {
		head = theHtml.indexOf("no-typhoon.png");
		if (0 <= head) {
			typhs.push("none");
		} else {
			alert("parseHTML failed.");
			return;
		}
	} 
		
	// 台風リストを更新
	reloadImg();
}


// 予報図の画像を更新
function reloadImg() {
	var imgURI;
	var imgType;
	var i;

	for (i = 0; i < typhs.length; i++) {
		if (typhs.length <= 1) {
 			imgURI = jmaURI + "img/no-typhoon.png";	// 台風情報がない場合の画像
		} else {
			imgType = (1 <= i)? "zoom" : "wide";
			imgURI = jmaURI + "images/" + imgType + "/" + typhs[i] + "-00.png";
			imgURI += "?" + new Date().getTime();  // 強制的にリロードさせるための小細工
		}
		LOG__("reloadImg URI: " + imgURI);
		document.getElementById("map" + i).setAttribute("src", imgURI);
	}
}


// 台風セレクタの表示
function dispTyphs() {
	var typhsField = document.getElementById("typhs");
	var child;
    var numReg = new RegExp("^[0-9][0-9]([0-9][0-9])$"); // 台風
    var alpReg = new RegExp("^([a-z])$"); // 熱帯低気圧
	var typhName;
	var i;

	while (typhsField.hasChildNodes())
		typhsField.removeChild(typhsField.lastChild);

	for (i = 0; i < typhs.length; i++) {
        typhName = null;
        
        // 台風のコードをセレクタ表示用の名前に変換
        if (typhName = (typhs[i].match(numReg) || [])[1]) {
			typhName -= 0;	// 文字列から数値に変換
        } else if (typhName = (typhs[i].match(alpReg) || [])[1]) {
			typhName = typhName;
		} else if (typhs[i] == "all") {
			typhName = "ALL";
        }
        
        // セレクタに台風を追加
        if (typhName) {
            child = document.createElement("div");
            child.innerText = typhName;
            child.id = "typh" + i;
            child.setAttribute("onclick", "mapDisp(" + i + ");");
            typhsField.appendChild(child);
        }
	}
}


function openWWW() {
	var uri;

	if (typhNum == 0)
		uri = jmaURI + "index.html";
	else
		uri = jmaURI + typhs[typhNum] + ".html";

	if (window.widget)
		widget.openURL(uri);
	else
		window.location.href = uri;
}

// リサイズ開始
function startDragging(event) {
	document.addEventListener("mousemove", mouseMove, true);
	document.addEventListener("mouseup", mouseUp, true);

	lastX = event.screenX;
	lastY = event.screenY;
}

// リサイズ中
function mouseMove(event) {
	var deltaX = 0;
	var deltaY = 0;

	// 最小サイズ制限
	if ((window.outerWidth + event.screenX - lastX < minWidth)
			|| (window.outerHeight + event.screenY - lastY < minHeight)) {
		window.resizeTo(minWidth, minHeight);
		resizeElements();
		return;
	}

	// 最大サイズ制限
	if ((maxWidth < window.outerWidth + event.screenX - lastX)
			|| (maxHeight < window.outerHeight + event.screenY - lastY)) {
		window.resizeTo(maxWidth, maxHeight);
		resizeElements();
		return;
	}

	deltaX = event.screenX - lastX;
	deltaY = event.screenY - lastY;

	// アスペクト比（4:3）固定
	if (window.outerHeight + deltaY - 60 < (window.outerWidth + deltaX - 30) * 0.75)
		deltaY = (window.outerWidth + deltaX - 30) * 0.75 - window.outerHeight + 60;
	else
		deltaX = (window.outerHeight + deltaY - 60) / 0.75 - window.outerWidth + 30;

	lastX = event.screenX;
	lastY = event.screenY;

	// ウインドウをリサイズ
	if (window.widget) {
		window.resizeBy(deltaX, deltaY);
		resizeElements();
	}
}

// リサイズ終了
function mouseUp(event) {
	document.removeEventListener("mousemove", mouseMove, true);
	document.removeEventListener("mouseup", mouseUp, true);

	// ウインドウサイズを設定ファイルに記録
	if (window.widget) {
		widget.setPreferenceForKey(window.outerWidth, "widgetWidth");
		widget.setPreferenceForKey(window.outerHeight, "widgetHeight");
	}
}

// 絶対値でウインドウをリサイズ
function setWidgetSize(theWidth, theHeight) {
	var width = theWidth;
	var height = theHeight;
	
	LOG__("setWidgetSize(" + theWidth +  ", " + theHeight + ")");
	
	// 横幅制限
	if (width != null) {
		if (width < minWidth)
			width = minWidth;
		if (maxWidth < width)
			width = maxWidth;
	} else {
		width = defaultWidth;
	}

	// 高さ制限
	if (height != null) {
		if (height < minHeight)
			height = minHeight;
		if (height < minHeight)
			height = minHeight;
	} else {
		height = defaultHeight;
	}

	if (window.widget) {
		window.resizeTo(width, height);
		resizeElements();
	}
}

// コンテンツの表示サイズをウインドウに合わせる
function resizeElements() {
	var width = window.outerWidth;
	var height = window.outerHeight;

	var front = document.getElementById("front");
	var mapFrame = document.getElementById("mapframe");
	var maps = document.getElementById("map").childNodes;
	var legend = document.getElementById("legend");
	var jma = document.getElementById("jma");
	var reload = document.getElementById("reload");
	var i;

	// front DIV 要素リサイズ
	front.style.width = width + "px";
	front.style.height = height + "px";

	// mapFrame DIV 要素リサイズ（480x360）
	mapFrame.style.width = width - 30 + "px";
	mapFrame.style.height = (width - 30) * 360 / 480 + "px";

	// map 画像リサイズ（480x424）
	for (i=0; i < maps.length; i++) {
		if (maps[i].nodeName == "IMG") {
			maps[i].style.width = width - 30 + "px";
			maps[i].style.height = (width - 30) * 424 / 480 + "px";
		}
	}

	// legend 画像リサイズ（460x64）
	legend.style.width = width - 30 - 20 + "px";
	legend.style.height = (width -  30 - 20) / 460 * 64 + "px";
	legend.style.bottom = 5 + "px";

	// ボタン位置 移動（legend のすぐ上）
	jma.style.bottom = 5 + parseInt(legend.style.height) + 4 + "px";
	reload.style.bottom = 5 + parseInt(legend.style.height) + 4 + "px";
}

// WWW ブラウザで widget 配布ページを表示
function goWidgetPage() {
	widget.openURL(widgetURI);
}


// ひっくりかえす
function showBack(event)
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToBack");

	front.style.display="none";
	back.style.display="block";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}

function showFront(event)
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");

	front.style.display="block";
	back.style.display="none";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}


// マウスオーバー時にボタン類を表示する
function onmousemoveFront(event) {
	document.getElementById("reload").style.display = "block";
	document.getElementById("jma").style.display = "block";
	document.getElementById("legend").style.display = "block";
	document.getElementById("growbox").style.display = "block";
}

function onmouseoutFront(event) {
	document.getElementById("reload").style.display = "none";
	document.getElementById("jma").style.display = "none";
	document.getElementById("legend").style.display = "none";
	document.getElementById("growbox").style.display = "none";
}


// デバッグ用
function LOG__(str) {
//	alert(str);
}
